Package: ggRandomForests
Type: Package
Title: Visually Exploring Random Forests
Version: 3.5.0
Date: 2026-08-04
Authors@R: person("John", "Ehrlinger",
  role = c("aut", "cre"),
  email = "john.ehrlinger@gmail.com",
  comment = c(ORCID = "0000-0002-5340-5154"))
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
URL: https://github.com/ehrlinger/ggRandomForests,
        https://ehrlinger.github.io/ggRandomForests/
BugReports: https://github.com/ehrlinger/ggRandomForests/issues
Description: Graphic elements for exploring Random Forests using the
    'randomForest' or 'randomForestSRC' package for survival, regression
    and classification forests and 'ggplot2' package plotting. Implements
    visualizations of the methods described in Breiman (2001)
    <doi:10.1023/A:1010933404324> and Ishwaran, Kogalur, Blackstone, and
    Lauer (2008) <doi:10.1214/08-AOAS169>.
Depends: R (>= 4.4.0)
Imports: randomForestSRC (>= 3.4.0), randomForest, varPro (>= 3.1.0),
        igraph, survival, tidyr, dplyr, ggplot2, patchwork, stringr
Suggests: bookdown, callr, covr, datasets, ggraph, kernelshap, knitr,
        lintr, magick, MASS, pkgdown, pkgload, quarto, ragg,
        RColorBrewer, rmarkdown, testthat, vdiffr
VignetteBuilder: quarto
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-08-04 14:21:39 UTC; ehrlinj
Author: John Ehrlinger [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5340-5154>)
Maintainer: John Ehrlinger <john.ehrlinger@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-04 17:00:07 UTC
Built: R 4.6.0; ; 2026-08-04 20:02:16 UTC; unix
