# Generated by roxygen2: do not edit by hand

S3method(autoplot,gg_brier)
S3method(autoplot,gg_error)
S3method(autoplot,gg_partial)
S3method(autoplot,gg_partial_rfsrc)
S3method(autoplot,gg_partialpro)
S3method(autoplot,gg_rfsrc)
S3method(autoplot,gg_roc)
S3method(autoplot,gg_survival)
S3method(autoplot,gg_variable)
S3method(autoplot,gg_vimp)
S3method(calc_roc,randomForest)
S3method(calc_roc,rfsrc)
S3method(gg_brier,rfsrc)
S3method(gg_error,randomForest)
S3method(gg_error,randomForest.formula)
S3method(gg_error,rfsrc)
S3method(gg_rfsrc,randomForest)
S3method(gg_rfsrc,rfsrc)
S3method(gg_roc,default)
S3method(gg_roc,randomForest)
S3method(gg_roc,rfsrc)
S3method(gg_survival,default)
S3method(gg_survival,rfsrc)
S3method(gg_variable,randomForest)
S3method(gg_variable,rfsrc)
S3method(gg_vimp,randomForest)
S3method(gg_vimp,rfsrc)
S3method(plot,gg_brier)
S3method(plot,gg_error)
S3method(plot,gg_partial)
S3method(plot,gg_partial_rfsrc)
S3method(plot,gg_partialpro)
S3method(plot,gg_rfsrc)
S3method(plot,gg_roc)
S3method(plot,gg_survival)
S3method(plot,gg_variable)
S3method(plot,gg_vimp)
S3method(print,gg_brier)
S3method(print,gg_error)
S3method(print,gg_partial)
S3method(print,gg_partial_rfsrc)
S3method(print,gg_partialpro)
S3method(print,gg_rfsrc)
S3method(print,gg_roc)
S3method(print,gg_survival)
S3method(print,gg_variable)
S3method(print,gg_vimp)
S3method(print,summary.gg)
S3method(summary,gg_brier)
S3method(summary,gg_error)
S3method(summary,gg_partial)
S3method(summary,gg_partial_rfsrc)
S3method(summary,gg_partialpro)
S3method(summary,gg_rfsrc)
S3method(summary,gg_roc)
S3method(summary,gg_survival)
S3method(summary,gg_variable)
S3method(summary,gg_vimp)
export(calc_auc)
export(calc_roc)
export(gg_brier)
export(gg_error)
export(gg_partial)
export(gg_partial_rfsrc)
export(gg_partialpro)
export(gg_rfsrc)
export(gg_roc)
export(gg_survival)
export(gg_variable)
export(gg_vimp)
export(kaplan)
export(nelson)
export(quantile_pts)
export(surv_partial.rfsrc)
export(varpro_feature_names)
importFrom(dplyr,all_of)
importFrom(dplyr,filter)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(ggplot2,.data)
importFrom(ggplot2,aes)
importFrom(ggplot2,autoplot)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,theme)
importFrom(parallel,mclapply)
importFrom(patchwork,wrap_plots)
importFrom(randomForest,randomForest)
importFrom(randomForestSRC,partial.rfsrc)
importFrom(randomForestSRC,vimp)
importFrom(stats,as.formula)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.response)
importFrom(stats,na.omit)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,xtabs)
importFrom(stringr,str_sub)
importFrom(survival,Surv)
importFrom(survival,strata)
importFrom(survival,survfit)
importFrom(tidyr,pivot_longer)
importFrom(utils,head)
importFrom(utils,tail)
