% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_varpro.R
\name{.build_varpro_imp_dfs}
\alias{.build_varpro_imp_dfs}
\title{Build the tidy importance data structures from importance() output}
\usage{
.build_varpro_imp_dfs(
  imp_out,
  imp_tree_mat,
  family,
  cutoff,
  nvar,
  faithful,
  local.std,
  conditional
)
}
\description{
Build the tidy importance data structures from importance() output
}
\keyword{internal}
