% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_varpro.R
\name{.build_varpro_imp_tree}
\alias{.build_varpro_imp_tree}
\title{Build per-tree importance matrix from varpro object results}
\usage{
.build_varpro_imp_tree(object)
}
\description{
Reconstructs the ntree x p per-tree importance matrix from
\code{object$results}, replicating the aggregation performed internally
by \code{varPro:::.importance.varpro.workhorse}.
}
\keyword{internal}
